import 'dart:typed_data';
import '../marker_handler/marker_data.dart';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_maps/maps.dart';
import 'dart:math';
import 'dart:convert';

/// A class containing thickness data for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.subDivID, this.thickness);

  final String subDivID;
  final int thickness;
}

/// ChoroplethMap is a stateful widget that contains a choropleth map.
/// The map is created using the Syncfusion Flutter Maps library and
/// coordinates fetched from the server.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,
    required this.relation,
    required this.measurements
  }) : super(key: key);

  final Uint8List relation;
  final List<Measurement> measurements;

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  late MapShapeSource mapShapeSource;
  late final MapZoomPanBehavior _zoomPanBehavior = MapZoomPanBehavior();
  List<IceThicknessModel> iceThicknessList = <IceThicknessModel>[];

  @override
  void initState() {
    super.initState();

    final Random random = Random();
    for (int i = 0; i <= 60; i++) {
      int randomNumber = random.nextInt(21); // 0 -> 20, NB: temp test data
      iceThicknessList.add(IceThicknessModel(i.toString(), randomNumber));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        SfMaps(
          layers: [
            MapShapeLayer(
              source: MapShapeSource.memory( // Map polygon
                widget.relation, // JSON coordinates from server
                shapeDataField: 'properties.SubDivID',
                dataCount: iceThicknessList.length,
                primaryValueMapper: (int index) => iceThicknessList[index].subDivID,
                shapeColorValueMapper: (int index) => iceThicknessList[index].thickness,
              ),
              color: Colors.blue.shade400, // Map color
              zoomPanBehavior: _zoomPanBehavior,
              strokeColor: Colors.black,
            ),
          ],
        ),
      ],
    );
  }
}

