import numpy as np
import laspy
import utm # src: https://github.com/Turbo87/utm
from itertools import groupby
from server.data_processing.area_processing import calculate_corners, define_gridareas


# hard coded files for test data
lazData_path = ["server/example_lidar_data/ot_N_000005_1.laz", "server/example_lidar_data/ot_N_000033_1.laz"]

# Info about data
with laspy.open(lazData_path[0]) as fh:
    # Print metadata properties
    print("File Version:", fh.header.version)
    print("Point Count:", fh.header.point_count)
    print("Scale Factors:", fh.header.scale)
    print("Offset:", fh.header.offset)

    print('Points from Header:', fh.header.point_count)
    las = fh.read()
    print(las)
    print('Points from data:', len(las.points))
    ground_pts = las.classification == 2
    bins, counts = np.unique(las.return_number[ground_pts], return_counts=True)
    print('Ground Point Return Number distribution:')
    for r, c in zip(bins, counts):
        print('    {}:{}'.format(r, c))

# check if lidar points is within range of the area selected
def inArea(position, areaRange):
    x, y, _ = position
    if (areaRange[0][0] > x > areaRange[1][0]) and (areaRange[0][1] < y < areaRange[1][1]):
        return True
    else:
        return False

# Calculation of height in an area from las files
def find_height(points):
    height_differences = [] # final container for height data

    # sort the points
    sorted_coords = sorted(points, key=lambda coord: (coord[0],coord[1]))

    # group the sorted points that has the same xy- coordinates together
    groupCoords = [list(group) for key, group in groupby(sorted_coords, key=lambda coord: (coord[0], coord[1]))]

    # loop through the groups to find the difference in height
    for group in groupCoords:
        if len(group) < 2 or group[0] == group[1]:
            continue # jump over iteration if there is only one coordinate or lidar registered the same point twice

        # find max and min height
        min_height = min(coords[2] for coords in group)
        max_height = max(coords[2] for coords in group)

        # difference between them
        difference = max_height - min_height
        height_differences.append(difference)

    # list of thickness in an area
    return height_differences

# find the height of an area based on the coordinates of it's center
# and it's affiliations (subId and groupId) (soon to be implemented
def calculate_area_data(center):
    # container for all the heights in area
    area_heights = []

    # set the limit of the area compared to local coordinates
    area_limit = calculate_corners(center[0],center[1], 1500)

    # Refactor lidar data to a readable format
    iceOver = laspy.read(lazData_path[0])
    iceUnder = laspy.read(lazData_path[1])

    # add two files together temporary test data(soon to be removed)
    ice_points = list(zip(iceOver.X,iceOver.Y,iceOver.Z)) + list(zip(iceUnder.X,iceUnder.Y,iceUnder.Z))
    max_point = max(ice_points)
    min_point = min(ice_points)

    # define all the sub-areas within the area, local coordinates
    grid_area_heights = define_gridareas(60,10, 1500,4)

    # find the heights of each sub-area => area-heights
    for sub_area in grid_area_heights:
        start = min(sub_area[2])
        end = max(sub_area[2])

        #test data
        # zone coordinates sett to be relative to the lidar data
        areazone = [(((min_point[0] - max_point[0]) * ((start[0]-center[0])/(area_limit[1][0]-area_limit[3][0]))) + (min_point[0] - max_point[0])/2 + max_point[0],
                     (((min_point[1] - max_point[1]) * ((start[1]-center[1])/(area_limit[1][1]-area_limit[3][1]))) + (min_point[1] - max_point[1])/2 + min_point[1])),
                    ((((min_point[0] - max_point[0]) * ((end[0]-center[0])/(area_limit[1][0]-area_limit[3][0]))) + (min_point[0] - max_point[0])/2 + max_point[0],
                     (((min_point[1] - max_point[1]) * ((end[1]-center[1])/(area_limit[1][1]-area_limit[3][1])))) + (min_point[1] - max_point[1])/2 + min_point[1]))]

        # filter data within sub-area zones
        points_in_area = list(filter(lambda point_position: inArea(point_position, areazone), ice_points))
        area_heights.append((sub_area[0],sub_area[1],find_height(points_in_area)))

    return area_heights



